################################################################################
# Function gpbi computes the General Purpose Biotic Index (GPBI) as described in
# Labrune et al. (2021).
#
# Olivier Gauthier, olivier.gauthier@univ-brest.fr, May 2021.
# 
# Usage
#           gpbi(x, refs, details = TRUE)
#
# Arguments:
#   x       : a matrix or data.frame containing the community data. rows are
#             sites/stations columns are species. x should contain species
#             absolute abondances.
#   refs    : either 
#               1) a numeric vector of the same length as the number of
#               rows in x with 0 indicating a tested station and any other
#               value a reference station.
#               2) a logical vector of the same length as the number of
#               rows in x with FALSE for a tested station and TRUE for a
#               reference station.
#               3) a numeric vector giving the indices of the rows of x that 
#               are reference stations. 
#
#   details : should the output include results of intermediate computations
#             (TRUE) or not (FALSE, the default).
#
# Output
# * if details is FALSE (default), a vector with the GPBI values for each station
#   in x, including the reference station(s).
# * if details is TRUE, a list containing the following elements
#
#   A        : a matrix with the A values (shared individuals). Rows are
#              reference stations and columns are compared stations.
#
#   C        : a matrix with the C values (lost individuals). Rows are
#              reference stations and columns are compared stations.
#
#   Rquant   : a matrix with the Rquant = A / (A+C) values. Rows are
#              reference stations and columns are compared stations.
#
#   per.loss : the mean Rquant value in each station; i.e. a vector with
#              the mean values of the columns of Rquant.
#
#   ref.bar  : if there are multiple reference stations, the mean of all the 
#              Rquant similarity matrix between the reference stations including
#              the diagonal; otherwise 1
#
#   GPBI     : a vector with the GPBI values for each station in x, including 
#              the reference station(s)
################################################################################
gpbi <- function(x, refs, details = FALSE) {
  x <- as.matrix(x)
  if (length(refs) == nrow(x)) {
    tmp <- as.logical(refs)
    refs <- which(tmp == TRUE)
  } else if (length(refs) > nrow(x)){
    stop("ref must have fewer or the same number of elements as there are rows in the community matrix (x).")
  }
  n.refs <- length(refs)  
  if (n.refs <= 0)
    stop("A minimum of one reference station is necessary to compute GPBI")
  n.tests <- nrow(x)
  ref.mat <- matrix(x[refs, ], nrow = n.refs)
  test.mat <- x
  if (n.refs == 1) {
    ref.bar <- 1
  } else {
    dgpi.refs <- numeric(0)
    for (i in 1:n.refs) {
      for (j in 1:n.refs) {
        A.refs <- apply(ref.mat[c(i,j), ], 2, min)
        C.refs <- ifelse(ref.mat[i, ] - A.refs > 0, ref.mat[i, ] - A.refs, 0)
        dgpi.refs <- c(dgpi.refs, sum(A.refs) / (sum(A.refs) + sum(C.refs)))
      }
    }
    ref.bar <- mean(dgpi.refs)
  }
  A.mat <- matrix(0, n.refs, n.tests)
  C.mat <- matrix(0, n.refs, n.tests)
  colnames(A.mat) <- colnames(C.mat) <- rownames(x)
  rownames(A.mat) <- rownames(C.mat) <- rownames(x)[refs]
  for (i in 1:n.refs) {
    for (j in 1:n.tests) {
      tmp <- rbind(ref.mat[i,], test.mat[j,])
      A <- apply(tmp, 2, min)
      C <- ifelse(tmp[1, ] - A > 0, tmp[1, ] - A, 0)
      A.mat[i,j] <- sum(A)
      C.mat[i,j] <- sum(C)
    }
  }
  Rquant <- A.mat/(A.mat + C.mat)
  per.loss <- colMeans(Rquant)
  GPBI <- 1 - abs(ref.bar - per.loss)
  if (details == TRUE) {
    res <- list(A = A.mat, C = C.mat, Rquant = Rquant, per.loss = per.loss, ref.bar = ref.bar, GPBI = GPBI)
  } else {
    res <- GPBI
  }
  return(res)
}
